#ifndef _ENVELOPE_H_INCLUDED
#define _ENVELOPE_H_INCLUDED

#include "Global.h"

class Envelope
{
public:
	Envelope();
	virtual ~Envelope();

	F_INLINE void setMode(int mode, int slope, int rate);
	F_INLINE void setHeight(int height);
	F_INLINE int getHeight(void);
	F_INLINE void updateHeight(void);

private:
	int Mode;        // 0:   1:
	int Slope;       // 0:jA 1:w
	int Rate;        // [g
	int curHeight;   // ݂̃Gx[vl

	int IntervalCnt; // C^[oJE^
	                 // [hؑ֎ɃC^[o錻ۂČׁA
	                 // NA0`0x7fffŃ[vB
};

F_INLINE void Envelope::setMode(int mode, int slope, int rate)
{
	Mode = mode;
	Slope = slope;
	Rate = rate;
}

F_INLINE void Envelope::setHeight(int height)
{
	curHeight = height;
}

F_INLINE int Envelope::getHeight(void)
{
	return curHeight;
}

F_INLINE void Envelope::updateHeight(void)
{
	IntervalCnt++;
	IntervalCnt &= 0x7fff;

	if(Mode == 0) // 
	{
		int shift = Rate >> 2;
		int exp = (Slope & (curHeight > 24575)) * 2; // w
		int interval = ~(0xffffffe0 << shift << exp) >> 16;

		if((IntervalCnt & interval) == 0 && Rate != 127) // [g127̏ꍇAωʂ0
		{
			static const int base[4] = {7, 6, 5, 4};
			int dHeight = (-2048 >> shift >> exp) * base[Rate & 3];
			curHeight -= dHeight;
			if(curHeight > 32767) curHeight = 32767;
		}
	}
	else // 
	{
		if(curHeight > 0)
		{
			int shift = Rate >> 2;
			int interval = ~(0xffffffe0 << shift) >> 16;

			if((IntervalCnt & interval) == 0 && Rate != 127) // [g127̏ꍇAωʂ0
			{
				static const int base[4] = {8, 7, 6, 5};
				int dHeight = (-2048 >> shift) * base[Rate & 3];
				if(Slope != 0)
					dHeight = (dHeight * curHeight) >> 15; // w
				curHeight += dHeight;
			}
		}
		else
			curHeight = 0;
	}
}

/*
F_INLINE void Envelope::updateHeight(void)
{
	if(Mode == 0) // 
	{
		static const int base[4] = {7, 6, 5, 4};
		int dHeight = (-2048 << 15) >> (Rate >> 2);
		if(Rate == 127) dHeight = 0; // [g127̏ꍇAωʂ0
//		dHeight >>= (Slope & (curHeight >= 24576)) * 2; // w
		if(Slope & (curHeight >= 24576))
			dHeight >>= 2; // wX[v
		dHeightAcc -= dHeight;
		curHeight += (dHeightAcc >> 15) * base[Rate & 3];
		if(curHeight > 32767) curHeight = 32767;
		dHeightAcc &= 0x7fff;
	}
	else // 
	{
		static const int base[4] = {-8, -7, -6, -5};
		int temp;
		int dHeight = (-2048 << 15) >> (Rate >> 2);
		if(Rate == 127) dHeight = 0; // [g127̏ꍇAωʂ0
		dHeightAcc -= dHeight;
		temp = (dHeightAcc >> 15) * base[Rate & 3];
		if(Slope != 0)
			temp = (temp * curHeight) >> 15; // w
		curHeight += temp;
		if(curHeight < 0) curHeight = 0;
		dHeightAcc &= 0x7fff;
	}
}
*/

#endif // #ifndef _ENVELOPE_H_INCLUDED
